
/* Chrysalide - Outil d'analyse de fichiers binaires
 * array.c - décompilation de l'opération récupérant la longueur d'un tableau
 *
 * Copyright (C) 2010-2013 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "translate.h"


#include "../../../decomp/expr/assign.h"
#include "../../../decomp/expr/dalvik/array.h"



/******************************************************************************
*                                                                             *
*  Paramètres  : instr = instruction d'origine à convertir.                   *
*                ctx   = contexte de la phase de décompilation.               *
*                                                                             *
*  Description : Décompile une instruction de type 'array-length'.            *
*                                                                             *
*  Retour      : Instruction mise en place ou NULL.                           *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDecInstruction *dalvik_decomp_instr_array_length(const GArchInstruction *instr, GDecContext *ctx)
{
    GDecInstruction *result;                /* Instruction à retourner     */
    vmpa_t addr;                            /* Adresse de l'instruction    */
    GArchOperand *operand;                  /* Opérande de l'instruction   */
    GDecInstruction *reg;                   /* Pseudo-registre redéfini    */
    GDecInstruction *len;                   /* Enregistrement de taille    */
    GDecInstruction *dest;                  /* Destination de la création  */

    g_arch_instruction_get_location(instr, NULL, NULL, &addr);

    operand = g_arch_instruction_get_operand(instr, 1);
    reg = g_dec_context_convert_register(ctx, operand, false, addr);
    len = g_dalvik_alength_new(G_DEC_EXPRESSION(reg));

    operand = g_arch_instruction_get_operand(instr, 0);
    dest = g_dec_context_convert_register(ctx, operand, true, addr);

    result = g_assign_expression_new(G_DEC_EXPRESSION(dest), G_DEC_EXPRESSION(len));

    return result;

}
